#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <windows.h>

#include "mach_include.h"
#include "mach.h"
#include "block.h"

#define PROGRAM_NAME "Over5"
#define SPLAT printf("*\n")
#define TESTBAUD 19200
#define MAGIC_SIZE 1000

struct O5Config o5config;


/*************************************************************************
**
** Panic (new cool with varargs)
**
******/
void panic(char *str, ...)
{
    va_list arglist;

    va_start(arglist,str);

    if (str) {
	printf(PROGRAM_NAME ": ");
	vprintf(str, arglist);
	puts("!");
    } else
	puts(PROGRAM_NAME ": panic!\n");

    va_end(arglist);

    exit(-1);
}

long getsize(FILE *f)
{
    long size, back;
    
    back=ftell(f);
    fseek(f, 0, SEEK_END);
    size=ftell(f);
    fseek(f, back, SEEK_SET);
    return size;
} 


int main (int argc, char **argv)
{
    DWORD t1, t0;
    int testtype;
    long us;
    char *errstr = "fel.";
    char serdevice[]="COM2";
    char *filebuf;
    FILE *fil;
    long filesize;

    if (argc < 2)
    {
      print_usage:
	printf (
	    "Usage: %s <test> [options]\n"
	    " where <test> is one of\n"
	    "  1 <us>    MicroWait() for <us> microseconds\n"
	    "  2 <str>   Send character string <str>\n"
	    "  3 <file>  Send <file>\n"
	    "  4 <file>  Receive to <file>\n"
	    "  5 <us>    Send break for <us> microseconds\n"
	    , argv[0]);
	return 1;
    }
    
    o5config.serdevice=serdevice;

    testtype = atoi(argv[1]);
    
    switch(testtype) {
    case 1:
	if (argc != 3) goto print_usage;
	us=atoi(argv[2]);
	t0=GetTickCount();
	MicroWait(us);
	t1=GetTickCount();
	printf ("Elapsed time: %f s\n", (t1-t0)/1000.0);
	return 0;

    case 2:
	if (argc != 3) goto print_usage;
	if ((errstr=CreateSerial())) goto print_error;
	Setbaud(TESTBAUD);
	SerWrite(argv[2], strlen(argv[2]));
	DeleteSerial();
	return 0;

    case 3:
	if (argc != 3) goto print_usage;
	if (!(fil=fopen(argv[2], "rb"))) goto print_error;
	filesize=getsize(fil);
	if (!(filebuf=malloc(filesize+1))) goto print_error;
	fread(filebuf, sizeof(char), filesize, fil); 
	fclose(fil);
	
	if ((errstr=CreateSerial())) goto print_error;
	Setbaud(TESTBAUD);
	SerWrite(filebuf, filesize);
	DeleteSerial();
	free(filebuf);
	return 0;
	
    case 4:
	if (argc != 3) goto print_usage;
	if (!(filebuf=malloc(MAGIC_SIZE))) goto print_error;

	if ((errstr=CreateSerial())) goto print_error;
	Setbaud(TESTBAUD);
	SerRead(filebuf, MAGIC_SIZE, 17);
	DeleteSerial();

	if (!(fil=fopen(argv[2], "wb"))) goto print_error;
	if (fwrite(filebuf, sizeof(char), MAGIC_SIZE, fil)==-1)
	    goto print_error;
	fclose(fil);
	free(filebuf);
	return 0;

    case 5:
	if (argc != 3) goto print_usage;
	us=atoi(argv[2]);
	if ((errstr=CreateSerial())) goto print_error;
	SendBreak();
	MicroWait(us);
	DeleteSerial();
	return 0;

    default:
	printf ("%d not implemented", testtype);
    }

  print_error:
    printf ("%s\n", errstr);
    return 1;
}

